import java.applet.*;
import java.awt.*;
import java.net.*;
import java.io.*;

public class EX19A extends Applet
{
    MainWinRes mainWin = null;
    Socket socket = null;
    static final int DICTIONARY_PORT = 2000;
    String host;

    public static void main(String args[])
    {   // Program being run as an application.
        EX19A applet = new EX19A();
        // Setup application frame for main window and start applet.
        Ex19aApplicationFrame frame = new Ex19aApplicationFrame(applet);
        applet.mainWin = frame.resource;
        applet.host = "199.42.65.2";
        applet.start();
    }

    public void init()
    {   // Program being run as applet.
        mainWin = new MainWinRes(this);
        mainWin.CreateControls();
        mainWin.IDC_TITLE.setFont(new Font("Helvetica", Font.BOLD, 18));
        // Use host that applet was served from.
        host = getCodeBase().getHost();
    }

    public void start()
    {   // Initialize screen controls.
        mainWin.IDC_STATUS.setText("");
        mainWin.IDC_RESULTS.setEditable(false);
    }

    public void stop()
    {   // Close down socket if it's active.
        if (socket != null)
        {
            try
                socket.close();
            catch (IOException e) {}
        }
    }

    public boolean handleEvent(Event event)
    {
        boolean retval = false;
        if (event.target == mainWin.IDC_LOOKUP)
        {   // Make sure the user has entered something.
            String word = mainWin.IDC_WORD.getText();
            if (word.length() == 0)
                mainWin.IDC_STATUS.setText("Please enter a word to lookup.");
            else
                lookupWord(word);
            retval = true;
        }
        return retval;
    }

    // This method does the actual lookup of the word by sending a request
    // across the network to the dictionary server.
    protected void lookupWord(String word)
    {
        mainWin.IDC_RESULTS.setText("");
        mainWin.IDC_STATUS.setText("Performing lookup...");

        try
        {   // Create socket and setup buffered input/output streams to communicate.
            socket = new Socket(host, DICTIONARY_PORT);
            DataInputStream in = new DataInputStream(new BufferedInputStream(socket.getInputStream()));
            PrintStream out = new PrintStream(new BufferedOutputStream(socket.getOutputStream()), true);
            mainWin.IDC_STATUS.setText("Sending word...");
            out.println("LOOKUP=" + word);    // Send lookup command to server.
            String line;

            mainWin.IDC_STATUS.setText("Waiting for response...");
            line = in.readLine();
            if (line.startsWith("DEFINITION="))
            {   // A definition was returned.
                // Strip off "DEFINITION=" response from first line of definition.
                line = line.substring(line.indexOf('=') + 1);
                // Read each line of definition from server.
                do
                    mainWin.IDC_RESULTS.appendText(line + "\n");
                while ((line = in.readLine()) != null);
                mainWin.IDC_STATUS.setText("Response received.  Closing connection.");
            }
            else if (line.startsWith("ERROR="))
            {   // An error occurred at the server.
                // Strip off "ERROR=" response from first line of response.
                String error = line.substring(line.indexOf('=') + 1);
                if (error.equalsIgnoreCase("WordNotFound"))
                    mainWin.IDC_STATUS.setText("The word '" + word + "' not found.  Try again.");
                else if (error.equalsIgnoreCase("DictionaryFileError"))
                    mainWin.IDC_STATUS.setText("Dictionary not available.  Try again later.");
                else
                    mainWin.IDC_STATUS.setText("Error looking up word.");
            }

            in.close();
            out.close();
            socket.close();
        }
        catch (UnknownHostException e1)
            mainWin.IDC_STATUS.setText("Unable to resolve dictionary server's name.");
        catch (IOException e2)
        {
            mainWin.IDC_STATUS.setText("Error opening connection to dictionary server.  Server may not be running.");
            if (socket != null)
            {   // Close down socket.
                try
                    socket.close();
                catch (IOException e) {}
            }
        }
        socket = null;
    }
}

// Frame used to wrap up main window when program is run as an application.
class Ex19aApplicationFrame extends Frame
{
    MainWinRes resource;
    MainMenuRes menu;
    Applet owner;

    public Ex19aApplicationFrame(Applet owner)
    {
        super("Internet Dictionary Client Application");
        this.owner = owner;

        // Since it's an application, give it a menu to allow user to exit.
        menu = new MainMenuRes(this);
        menu.CreateMenu();

        setFont(new Font("Dialog", Font.PLAIN, 8));
        resource = new MainWinRes(this);
        resource.CreateControls();
        resource.IDC_TITLE.setFont(new Font("Helvetica", Font.BOLD, 18));
        setResizable(false);    // Do not allow frame to be resized.
        show();
    }

    public boolean handleEvent(Event event)
    {   // Intercept frame messages and pass the others along to the applet.
        if (event.target == menu.ID_FILEEXIT || event.id == Event.WINDOW_DESTROY)
        {
            dispose();
            System.exit(0);
            return true;
        }
        else
            return owner.handleEvent(event);
    }
}
