import java.awt.*;
import java.io.*;
import java.net.*;

public class EX19B extends Frame implements Runnable, LookupInterface
{
    static final int DICTIONARY_PORT = 2000;
    // Counts used to track server statistics.
    int currentConnections = 0;
    int totalConnections = 0;
    int wordsFound = 0;
    int wordsNotFound = 0;
    // Interface objects.
    MainMenuRes menu;
    MainWindowRes mainWin;
    // Server's primary thread and socket.
    Thread running = null;
    ServerSocket listen = null;

    public static void main(String args[])
    {
        new EX19B("Internet Dictionary Server");
    }

    public EX19B(String caption)
    {
        super(caption);

        // Create main menu built using Resource Wizard.
        menu = new MainMenuRes(this);
        menu.CreateMenu();

        // Setup font to use.
        setFont(new Font("Dialog", Font.PLAIN, 8));
        mainWin = new MainWindowRes(this);
        mainWin.CreateControls();

        // Initialize main window fields.
        mainWin.IDC_CURRENTCONNECTIONS.setText("0");
        mainWin.IDC_TOTALCONNECTIONS.setText("0");
        mainWin.IDC_WORDSFOUND.setText("0");
        mainWin.IDC_WORDSNOTFOUND.setText("0");

        setResizable(false);    // Do not allow frame to be resized.
        show();

        // Start up thread to listen for clients.
        running = new Thread(this);
        running.start();
    }

    public boolean handleEvent(Event event)
    {
        boolean retval = false;

        if (event.target == menu.ID_FILE_EXIT || event.id == Event.WINDOW_DESTROY)
        {   // Kill server thread if running.
            if (running != null)
                running.stop();
            dispose();
            System.exit(0);
            retval = true;
        }
        return retval;
    }

    public void run()
    {
        mainWin.IDC_ACTIVITY.addItem("Starting server socket...");

        try
        {   // Create primary thread.
            listen = new ServerSocket(DICTIONARY_PORT);
            while (true)
            {   // Wait for connection requests and spin off Lookup threads 
                // to service each request.
                Socket socket = listen.accept();
                new Lookup(this, socket);
            }
        }
        catch (IOException e)
            mainWin.IDC_ACTIVITY.addItem("Server socket error: " + e);

        // Close down the socket.
        try
            listen.close();
        catch (IOException e) {}
        listen = null;
    }

    // Called by each lookup thread to update server.
    public synchronized void connectionUpdate(String event)
    {
        mainWin.IDC_ACTIVITY.addItem(event);
    }

    // Called by each lookup thread to indicate it has started.
    public synchronized void connectionStart()
    {
        mainWin.IDC_CURRENTCONNECTIONS.setText(String.valueOf(++currentConnections));
        mainWin.IDC_TOTALCONNECTIONS.setText(String.valueOf(++totalConnections));
    }

    // Called by each lookup thread to indicate it has stopped.
    public synchronized void connectionStop()
    {
        mainWin.IDC_CURRENTCONNECTIONS.setText(String.valueOf(--currentConnections));
    }

    // Called by each lookup thread to indicate that the requested word has been found.
    public synchronized void wordFound()
    {
        mainWin.IDC_WORDSFOUND.setText(String.valueOf(++wordsFound));
    }

    // Called by each lookup thread to indicate that the requested word has not been found.
    public synchronized void wordNotFound()
    {
        mainWin.IDC_WORDSNOTFOUND.setText(String.valueOf(++wordsNotFound));
    }
}
