import java.io.*;
import java.net.*;

public class Lookup extends Thread
{
    static final String DICTIONARY_FILE = "dictionary.dat";
    // Interface used to udpate server with lookup events.
    LookupInterface lookupUser;
    // Socket accepted from server to communicate with client.
    Socket socket;
    // Input/output streams connected to socket.
    DataInputStream in;
    PrintStream out;

    public Lookup(LookupInterface lookupUser, Socket socket)
    {
        this.lookupUser = lookupUser;
        this.socket = socket;
        start();    // Kick-off thread.
    }

    public void run()
    {
        lookupUser.connectionStart();
        try
        {   // Setup buffered streams to read/write over socket.
            in = new DataInputStream(new BufferedInputStream(socket.getInputStream()));
            out = new PrintStream(new BufferedOutputStream(socket.getOutputStream()), true);
            String request;

            while ((request = in.readLine()) != null)
            {   // Read until a LOOKUP command is received.
                if (request.startsWith("LOOKUP="))
                {   // Lookup word and then drop out of thread.
                    wordSearch(request.substring(7));
                    break;
                }
            }
        }
        catch (IOException e)
            lookupUser.connectionUpdate("I/O Error on socket: " + e);

        // Close down the socket.
        try
            socket.close();
        catch (IOException e) {}

        lookupUser.connectionStop();
    }

    protected void wordSearch(String word)
    {
        try
        {   // Create a buffered input stream to search dictionary file for word.
            DataInputStream dis = new DataInputStream(new BufferedInputStream(new FileInputStream(DICTIONARY_FILE)));
            String line;
            boolean found = false;

            while (!found && (line = dis.readLine()) != null)
            {   // Keep reading until word is found or end of file is reached.
                int idx;
                if (line.startsWith("^") &&  (idx = line.indexOf('^', 1)) > 0)
                {   // Format is "^Word^Definition...".  Since definitions can occupy
                    // multiple lines in file, "^" indicator is used to start each word
                    // and definition sequence.
                    String toMatch = line.substring(1, idx);
                    if (word.equalsIgnoreCase(toMatch) && word.length() == toMatch.length())
                    {   // Build "DEFINITION" response to be sent back to client.
                        line = "DEFINITION=" + line.substring(idx + 1);
                        // Keep reading and sending lines until the end of file is reached
                        // or the next word in the file is reached.
                        do
                            out.println(line);
                        while ((line = dis.readLine()) != null && !line.startsWith("^"));
                        found = true;
                        lookupUser.wordFound();
                    }
                }
            }
            if (!found)
            {   // Send "ERROR" response back to client indicating word was not found.
                out.println("ERROR=WordNotFound");
                lookupUser.wordNotFound();
            }
        }
        catch (FileNotFoundException fe)
        {   // Update client and server.
            out.println("ERROR=DictionaryFileError");
            lookupUser.connectionUpdate("Dictionary file not found.");
        }
        catch (IOException e)
        {   // Update client and server.
            out.println("ERROR=DictionaryFileError");
            lookupUser.connectionUpdate("I/O Error reading dictionary file.");
        }
    }
}